<?php

class com_bul7_wp_plugin_FacebookCommentImporter {
    private $refreshInterval = 300;

    public function __construct() {
        $this->readSettings();
        add_action('wp_head', array($this, 'import'));
        add_action('admin_menu', array(&$this, 'registerAdminMenuOptions'));
    }

    function registerAdminMenuOptions() {
        add_options_page('Facebook Comment to WordPress', 'FB2WP Comments', 'administrator', get_class($this), array(&$this, 'adminSettingsPage'));
    }

    public function import() {
        if (!is_single()) return;
        $helper = new com_bul7_wp_plugin_FacebookCommentImporterHelper(get_the_ID());
        $isNeedRefresh = $helper->lastImport + $this->refreshInterval <= time();
        if (!$isNeedRefresh) return;

        $url = get_permalink();
        $comments = current($this->fetchCommentsFor($url));
        if (is_array($comments) && is_array($comments['data']))
            $helper->import($comments['data']);
    }

    public function adminSettingsPage() {
        $tokenField = get_class($this).'_submit';
        $isSubmitted = isset($_POST[$tokenField]) && $_POST[$tokenField] == 'Y';

        $messages = array();
        $vars = array();
        if ($isSubmitted) {
            if ($_POST['do_delete_facebook_comments']) {
                $this->deleteAllImportedFacebookComments();
                $messages[] = 'All imported Facebook comments removed.';
            } // do_delete_facebook_comments

            else if ($_POST['do_update_settings']) {
                if (isset($_POST['refreshInterval']) && is_numeric($_POST['refreshInterval']) && $_POST['refreshInterval'] >= 0)
                    $this->refreshInterval = intval($_POST['refreshInterval']);
                else
                    $messages[] = 'WARNING: Refresh Interval not updated. Must be numeric greater or equal to 0. ' . $_POST['refreshInterval'] . ' given';
                $this->saveSettings();
                $messages['Settings saved'];
            }
        }
        $vars['refreshInterval'] = $this->refreshInterval;
        $vars['numFacebookComments'] = $this->countFacebookComments();
        
        $this->showAdminPage($tokenField, $vars, $messages);
    }

    private function countFacebookComments() {
        global $wpdb;
        /* @var $wpdb wpdb */

        if (FALSE === $wpdb->query("SELECT COUNT(*) cnt FROM {$wpdb->prefix}comments WHERE comment_agent LIKE 'facebook-seo-comments%'"))
            throw new Exception('countFacebookComments: ' . mysql_error());
        return $wpdb->last_result[0]->cnt;
    }

    private function showAdminPage($tokenField, $vars, $messages) {
        extract($vars);
?>
    <div class="wrap">
    <h2>Facebook Comments to WordPress</h2>
        <?php if ($messages) { ?>
            <div>
                <ul>
                    <?php foreach ($messages as $m) { echo "<li>$m</li>"; } ?>
                </ul>
            </div>
        <?php } // messages ?>
        <form method="POST">
            <input type="hidden" name="<?php echo $tokenField; ?>" value="Y" />
            <p>Number of imported comments: <?php echo $numFacebookComments; ?> <input type="submit" name="do_delete_facebook_comments" value="Delete All Imported Facebook Comments" /></p>
            <p><input type="submit" name="do_refresh" value="Refresh" /></p>
            <h3>Facebook Comments Import Settings</h3>
            <p>Refresh Interval: <input type="text" name="refreshInterval" size="60" value="<?php echo $refreshInterval; ?>" /></p>
            <p><input type="submit" name="do_update_settings" value="Update Settings" /></p>
        </form>
    </div>
<?php
    }

    private function readSettings() {
        $options = get_option(get_class($this));
        if (!is_array($options)) $options = array();
        if (!isset($options['refreshInterval'])) $options['refreshInterval'] = 300;

        $this->refreshInterval = $options['refreshInterval'];
    }

    private function saveSettings() {
        $options = array(
            'refreshInterval' => $this->refreshInterval
        );
        update_option(get_class($this), $options);
    }

    private function deleteAllImportedFacebookComments() {
        global $wpdb;
        $wpdb->query("DELETE FROM `wp_comments` WHERE `comment_agent` = 'facebook-seo-comments-1.0'");
        $wpdb->query("DELETE FROM `wp_postmeta` WHERE `meta_key` LIKE  '%FBSEOCommentsMap%'");
    }

    private function fetchCommentsFor($permalink) {
        // $permalink = "http://gotvarstvo.georgievi.net/a/cvetnica";
        $request_url = "https://graph.facebook.com/comments/?ids=" .
                urlencode($permalink) . '&locale=bg_BG';
        $requests = @file_get_contents($request_url);
        
        if ($requests === FALSE) return array();

        return json_decode($requests, TRUE);
    }
}

class com_bul7_wp_plugin_FacebookCommentImporterHelper {
    private $postId;
    private $commentMap;
    public $lastImport;

    public function __construct($postId) {
        $this->postId = $postId;
        $this->readPostMeta();
    }

    public function import($comments) {
        $this->importComments($comments);
        $this->savePostMeta();
    }

    private function importComments($comments, $parentCommentId = NULL) {
        foreach ($comments as $num => $comment) {
            $this->importComment($comment, $parentCommentId);
        }
    }

    private function importComment($comment, $parentFBId) {
        if (!isset($this->commentMap[$comment['id']])) {
            $db = $this->getDb();

            $createdDateTime = new DateTime($comment['created_time']);

            $timeOffsetFromGmt = -intval($createdDateTime->getOffset());
            $createdDateTime->modify($timeOffsetFromGmt.' seconds');
            $dateGmt = $createdDateTime->format('Y-m-d H:i:s');

            $timeOffsetToLocal = date('Z');
            $createdDateTime->modify($timeOffsetToLocal.' seconds');
            $date = $createdDateTime->format('Y-m-d H:i:s');

            $data = array(
                'comment_post_ID' => $this->postId,
                'comment_author' => $comment['from']['name'],
                'comment_author_email' => '',
                'comment_author_url' => 'http://www.facebook.com/profile.php?id='.$comment['from']['id'],
                'comment_date' => $date,
                'comment_date_gmt' => $dateGmt,
                'comment_content' => $comment['message'],
                'comment_karma' => 0,
                'comment_approved' => 1,
                'comment_agent' => 'facebook-seo-comments-1.0',
                'comment_type' => '',
                'comment_parent' => is_null($parentFBId) ? 0 : $this->commentMap[$parentFBId]['id'],
                'user_id' => 0
            );
            if ($db->insert($db->prefix.'comments', $data)) {
                $this->commentMap[$comment['id']]['id'] = $db->insert_id;
            } else {
                return;
            }
        }
        if (isset($comment['comments']['data']))
            $this->importComments($comment['comments']['data'], $comment[id]);
    }

    private function readPostMeta() {
        $metaEncoded = get_metadata('post', $this->postId, 'FBSEOCommentsMap', true);
        $meta = ($metaEncoded == '') ? array() : json_decode($metaEncoded, TRUE);

        $this->lastImport = isset($meta['lastImport']) ? $meta['lastImport'] : 0;
        $this->commentMap = isset($meta['map']) ? $meta['map'] : array();
    }

    private function savePostMeta() {
        $meta = array(
            'lastImport' => time(),
            'map' => $this->commentMap
        );
        update_metadata('post', $this->postId, 'FBSEOCommentsMap', json_encode($meta));
    }

    /**
     * @global wpdb $wpdb
     * @return wpdb
     */
    private function getDb() {
        global $wpdb;
        return $wpdb;
    }

}


?>
